/*
  Tutanchamun, an arcade game
  Copyright (C) 1998  David Kastrup
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
#include <stdio.h>
#include <sys/time.h>
/*#include <termios.h>*/
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include "tutan.h"
#include "tutdisp.h"
#include "tutsound.h"

#define NULLSTATE 128
#define STARTSTATE 0
#define OTHERSTATE 255

static int state;
static long carry;
static FILE *fpsound;

inline void toggle()
{
  state ^= STARTSTATE^OTHERSTATE;
}

inline void guzzle(int i) {
  carry += i;
  if (fpsound) {
    while (carry >= 500) {
      carry -= 500;
      putc(state, fpsound);
    }
  }
}

void startsound()
{
  catchup();
  carry = 0;
  state = STARTSTATE;
}

void stopsound()
{
  struct timeval del;

  if (fpsound) {
    putc(NULLSTATE, fpsound);
    fflush(fpsound);
    ioctl(fileno(fpsound),SNDCTL_DSP_SYNC,0);
  } else {
    del.tv_usec = (carry % 4000000)/4;
    del.tv_sec = carry/4000000;
    d_sleep(&del);
  }
  startclock();
}

void opensound()
{
  fpsound = fopen("/dev/dsp","w");
}

void closesound()
{
  if (fpsound) {
    fclose(fpsound);
    fpsound = NULL;
  }
}

void sndfrs()
{
  unsigned char b,d=0,e=0;
  startsound();
  for (b=15; b!=0; ) {
    if (--d == 0) {
      toggle();
      d = b;
      guzzle(86);
    } else
      guzzle(16);
    if (--e != 0) {
      guzzle(510);
      continue;
    }
    toggle();
    guzzle(110);
    e = b-- + 35;
  }
  stopsound();
}

unsigned char loctab[] = {
  3,
  35, 2, 0, 0x82,
  26,10,0,2,
  35,2,0,0x82,
  26,10,0,2,
  35,2,0,0x82,
  26,6,0,1,
  52,2,0,4,
  52,1,0,0x81,
  52,12+0x80};

unsigned char wintab[] = {
  10,
  35,1,0,0x81,37,1,35,0x81,31,1,37,0x81,
  35,1,0,0x81,39,1,41,0x81,46,1,39,0x81,
  41,1,46,0x81,52,1,41,0x81,46,1,52,0x81,
  46,0x82,69,1,0,0x81,69,1,0,0x81,
  35,1,0,0x81,37,1,35,0x81,31,1,37,0x81,
  35,1,0,0x81,39,1,41,0x81,46,1,39,0x81,
  41,1,46,0x81,52,1,41,0x81,46,1,55,0x81,
  52,0x82,52,0x82,52,0x82};

static unsigned char tmtab[] = {2, 13, 1, 0, 0xc3},
  keytab[] = {12, 35, 1, 26, 1, 41, 1, 39, 1, 35, 0x82},
  bontab[] = { 12, 35, 1, 37, 1, 35, 1, 26, 1, 35, 1, 41, 1,
	       39, 1, 37, 1, 35, 1, 52, 0x83},
  deatab[] = { 20, 83, 1, 0, 1, 62, 2, 55, 2, 52, 2, 46, 2, 55, 3,
	       62, 1, 66, 1, 0, 1, 55, 1, 0, 1, 62, 3, 55, 1, 52, 1,
	       0, 1, 46, 1, 0, 1, 41, 3, 0, 1, 41, 1, 0, 0x81},
  ovrtab[] = { 20, 41, 1, 0, 1, 41, 3, 46, 1, 52, 1, 0, 1, 41, 1, 0,
	       1, 46, 3, 52, 1, 55, 1, 0, 1, 46, 1, 0, 1, 52, 2, 62,
	       2, 55, 2, 66, 2, 62, 3, 0, 1, 62, 2, 0, 0x81 },
  platab[] = { 16, 69, 1, 0, 1, 52, 2, 46, 2, 41, 2, 39, 2, 35, 4,
	       41, 2, 52, 2, 31, 2, 39, 2, 46, 2, 35, 2, 52, 0x83},
  higtab[] = { 20, 83,0x81,62,0x81,83,0x81,62,0x81,52,0x81,62,0x81,
	       52,0x81,41,0x81,52,0x81,39,0x81,41,0x81,52,0x81,62,0x81,
	       55,0x81,66,0x81,52,0x81,55,0x81,66,0x81,83,0x81,62,0x81,
	       83,0x81,52,0x81,55,0xc2 },
  frmtab[] = { 6,9,1,0,1,9,1,0,1,9,1,0,1,9,0x81 };
  
static unsigned char *posit, *posit0, dura;

void musini(unsigned char *p)
{
  dura = *(posit = posit0 = p);
}

void sndfrm()
{
  musini(frmtab);
}

void sndtim()
{
  musini(tmtab);
}

void sndwin()
{
  musini(wintab);
}

void sndloc()
{
  musini(loctab);
}

void sndkey()
{
  musini(keytab);
}

void sndbon()
{
  musini(bontab);
}

void snddea()
{
  musini(deatab);
}

void sndhig()
{
  musini(higtab);
}

void sndpla()
{
  musini(platab);
}

void sndovr()
{
  musini(ovrtab);
}

void music()
{
  unsigned char d,e,b;
  unsigned ec;

  startsound();
  do {
    d = *++posit;
    e = *++posit;
    ec = dura * (e & 0x3f) * 256;
    
    b=d;
    state = d ? STARTSTATE : NULLSTATE;
    for (;ec;--ec) {
      if (d && --b == 0) {
	b = d;
	toggle();
      }
      guzzle(147);
    }
    if (e & 0x40)
      posit = posit0;
  } while (! (e&0x80));
  stopsound();
}
